//
//  LSItemViewController.m
//  sdkdemo
//
//  Created by William Meharg on 2012-10-21.
//  Modified by Anthony Yu
//  Copyright (c) 2012 CA Technologies. All rights reserved.
//

#import "LSItemViewController.h"
#import "L7SAFJSONRequestOperation.h"
#import "L7SHTTPClient.h"



@interface LSItemViewController ()

@end

@implementation LSItemViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.items = [NSMutableArray array];
}

- (void)viewWillAppear:(BOOL)animated  {
    [super viewWillAppear:animated];
    self.navigationItem.leftBarButtonItem.title = @"Back";
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [self loadItems];
}

- (void)viewDidDisappear:(BOOL)animated {
    [self.items removeAllObjects];
}

- (IBAction)reloadItems:(id)sender {
    [self loadItems];
}


- (void)loadItems {
    
    [self.items removeAllObjects];
    [self.tableView reloadData];
    
    L7SHTTPClient *httpClient = [[ L7SHTTPClient alloc] initWithBaseURL:[NSURL URLWithString:@"https://baseURL:8443"]];


    [httpClient registerHTTPOperationClass:[L7SAFJSONRequestOperation class]];
 
    [httpClient setDefaultHeader:@"Accept" value:@"application/json"];
     NSMutableDictionary * parameters = [[NSMutableDictionary alloc] init];
    
    [parameters setObject:@"listProducts" forKey:@"operation"];
    
    [httpClient getPath:@"/protected/resource/products" parameters:parameters success:^(L7SAFHTTPRequestOperation *operation, id responseObject) {
        
        [self.items removeAllObjects];
        
        NSLog(@"Result %@", responseObject);
        
        for (id item in [responseObject objectForKey:@"products"] ) {
            NSLog(@"item %@", item);
            NSString *name = [item objectForKey:@"name"];
            if (name)
                [self.items addObject:name];
        }
        
        [self.tableView reloadData];
   
        
    } failure:^(L7SAFHTTPRequestOperation *operation, NSError *error) {

        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error"
                                                        message:[[error userInfo] objectForKey:NSLocalizedDescriptionKey]
                                                       delegate:nil
                                              cancelButtonTitle:@"OK"
                                              otherButtonTitles:nil];
        [alert show];
        NSLog(@"Error %@", error);

    }];
  
    
  }

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return self.items.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"item";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    cell.textLabel.text = [self.items objectAtIndex:indexPath.row];
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
}

@end
