//
//  LSMainTableViewController.m
//  sdkdemo
//
//  Created by William Meharg on 2012-10-21.
//  Modified by Anthony Yu
//  Copyright (c) 2012 CA Technologies. All rights reserved.
//

#import "LSMainTableViewController.h"
#import "L7SClientManager.h"
#import "L7SErrors.h"
#import <AVFoundation/AVFoundation.h>

@interface LSMainTableViewController () <AVCaptureMetadataOutputObjectsDelegate>{
    BOOL _isScanning;
    BOOL _isScanned;
}

@property (nonatomic, strong) AVCaptureSession *session;
@property (nonatomic, strong) AVCaptureVideoPreviewLayer *previewLayer;

@end

@implementation LSMainTableViewController

enum {
    kSectionIndexWarehouse = 0,      
    kSectionIndexAuthentication,
    kSectionCount
};


enum {
   // kAuthenticationLoginDevice = 0,
    kAuthenticationLogoutDevice,
    //kAuthenticationLogonApp,
    kAuthenticationLogoffApp,
    kAuthenticationDeregister,
    kAuthenticationSSOEnabled,
    //kAuthenticationOptionsCount
};


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        
    }
    

    return self;
}

- (void)reloadView{
    [self.tableView reloadData];
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(reloadView)
                                                 name:UIApplicationDidBecomeActiveNotification
                                               object:nil];

    [[L7SClientManager sharedClientManager] setDelegate:self];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(didReceiveStatusUpdate:)
                                                 name:L7SDidReceiveStatusUpdateNotification
                                               object:nil];

    if([[L7SClientManager sharedClientManager] isSSOEnabled]){
        [self.singleSignOnSwitch setOn:YES animated:YES];
    }else{
        [self.singleSignOnSwitch setOn:NO animated:YES];
    }
    

   
    self.navigationItem.title = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleDisplayName"];

    
    
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}



- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath{
    
    switch (indexPath.section) {
        case kSectionIndexAuthentication:
            switch (indexPath.row) {
                case kAuthenticationLogoutDevice:{
                    if([[L7SClientManager sharedClientManager] isSSOEnabled]){
                        if([[L7SClientManager sharedClientManager] isDeviceLogin]){
                            cell.accessoryType = UITableViewCellAccessoryNone;
                            cell.textLabel.text = @"Logout Device";
                            cell.textLabel.enabled = YES;
                        }else{
                            //cell.accessoryType = UITableViewCellAccessoryCheckmark;
                            cell.textLabel.text = @"Device logged out";
                            cell.textLabel.enabled = NO;
                        }
                    }else{
                        cell.textLabel.enabled = NO;
                        cell.accessoryType = UITableViewCellAccessoryNone;
                        cell.textLabel.text = @"Device logged out";
                    }

                }break;
                case kAuthenticationLogoffApp:{
                    if(![[L7SClientManager sharedClientManager] isAppLogon]){
                        //cell.accessoryType = UITableViewCellAccessoryCheckmark;
                        cell.textLabel.text = @"App logged off";
                        cell.textLabel.enabled = NO;
                    }else{
                        cell.accessoryType = UITableViewCellAccessoryNone;
                        cell.textLabel.text = @"Logoff App";
                        cell.textLabel.enabled = YES;
                    }
                }break;
                case kAuthenticationDeregister:{
                    if(![[L7SClientManager sharedClientManager] isRegistered]){
                        //cell.accessoryType = UITableViewCellAccessoryCheckmark;
                        cell.textLabel.text = @"Device unregistered";
                        cell.textLabel.enabled = NO;
                    }else{
                        cell.accessoryType = UITableViewCellAccessoryNone;
                        cell.textLabel.text = @"De-Register";
                        cell.textLabel.enabled = YES;
                    }
                }break;
                default:
                    break;
            }
            break;
            
        default:
            break;
    }

    
 
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    [cell setSelected:NO];

    
    switch (indexPath.section) {
        case kSectionIndexAuthentication:
            switch (indexPath.row) {
                case kAuthenticationLogoutDevice:{
                    if([[L7SClientManager sharedClientManager] isDeviceLogin]){
                        [[L7SClientManager sharedClientManager] logoutDevice];
                    }
                }break;
                case kAuthenticationLogoffApp:{
                    if([[L7SClientManager sharedClientManager] isAppLogon]){
                       [[L7SClientManager sharedClientManager] logoffApp];
                    }
                }break;
                case kAuthenticationDeregister:{
                    if([[L7SClientManager sharedClientManager] isRegistered]){
                        [[L7SClientManager sharedClientManager] deRegister];
                    }
                }break;
        
                default:
                    break;
            }
            break;
            
        default:
            break;
    }
    
    [self.tableView reloadData];

}


- (void) didReceiveStatusUpdate:(NSNotification *) notification{
    NSLog(@"receive status update notification %@", notification);
    
    L7SClientState state = (L7SClientState)[[notification.userInfo objectForKey:L7SStatusUpdateKey] intValue];
    
    switch (state) {
        case L7SDidFinishRegistration:{
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Device Registered"
                                                            message:@"Device registered"
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            
            [self reloadView];
        }break;
        case L7SDidFinishAuthentication:{
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Authentication"
                                                            message:@"Authenticated"
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            
            [self reloadView];

        }break;
        case L7SDidFinishDeRegistration:{
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Device De-Registered"
                                                            message:@"Please kill the app to achieve a clean device reset before any other activities"
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            
            [self reloadView];
        }
        case L7SDidLogout:{
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Device Logout"
                                                            message:@"Device logged out"
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            
            [self reloadView];
        }break;
        case L7SDidLogoff:{
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"App logoff"
                                                            message:@"App logged off"
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            
            [self reloadView];
        }break;
        case L7SQRAuthenticationPollingStopped:{
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"QR Authentication"
                                                            message:@"QR Authentication Stopped"
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            
            
        }break;
        default:
            break;
    
    }
}



- (void)DidReceiveError:(NSError *)error{
    
    switch([error code]) {
        case L7SAuthenticationJWTError:
        {
                     
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[[error userInfo] objectForKey:NSLocalizedFailureReasonErrorKey]
                                                            message:[[error userInfo] objectForKey:NSLocalizedDescriptionKey]
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            break;
        }
        case  L7SAuthenticationLogoffError:
        {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[[error userInfo] objectForKey:NSLocalizedFailureReasonErrorKey]
                                                            message:[[error userInfo] objectForKey:NSLocalizedDescriptionKey]
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            break;
        }
        default:
        {
            //just show all errors for testing puporse
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[[error userInfo] objectForKey:NSLocalizedFailureReasonErrorKey]
                                                            message:[[error userInfo] objectForKey:NSLocalizedDescriptionKey]
                                                           delegate:nil
                                                  cancelButtonTitle:@"OK"
                                                  otherButtonTitles:nil];
            [alert show];
            break;
        }
        break;
    }
    
    
}
- (IBAction)singleSignOnSwitchPushed:(id)sender {
    if([(UISwitch *) sender isOn]){
        [[L7SClientManager sharedClientManager] enableSingleSignOn:YES];
    }else{
        [[L7SClientManager sharedClientManager] enableSingleSignOn:NO];
    }
    [self reloadView];
    
}

- (IBAction)scanQRCodeButtonPressed:(id)sender {
    NSLog(@"TBD: scan barcode here...");
    if([[L7SClientManager sharedClientManager] isDeviceLogin] && !_isScanning){
        _session = [[AVCaptureSession alloc] init];
        AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
        NSError *error = nil;
        
        AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice:device
                                                                            error:&error];
        if (input) {
            [_session addInput:input];
        } else {
            NSLog(@"Error: %@", error);
        }
        
        AVCaptureMetadataOutput *output = [[AVCaptureMetadataOutput alloc] init];
        [output setMetadataObjectsDelegate:self queue:dispatch_get_main_queue()];
        [_session addOutput:output];
        [output setMetadataObjectTypes:@[AVMetadataObjectTypeQRCode]];
        
        _previewLayer = [AVCaptureVideoPreviewLayer layerWithSession:_session];
        
        // Display full screen
        _previewLayer.frame = CGRectMake(0.0, 0.0, self.view.frame.size.width, self.view.frame.size.height);
        
        // Add the video preview layer to the view
        [self.view.layer addSublayer:_previewLayer];
        
        [_session startRunning];
        _isScanning = YES;
        [_scanCodeButton setTitle:@"Cancel"];
    }else if (_isScanning){
        [_session stopRunning];
        [_previewLayer removeFromSuperlayer];
        _session = nil;
        _previewLayer = nil;
        [_scanCodeButton setTitle:@"Scan Code"];
        _isScanning = NO;
    }
    

}


- (void)captureOutput:(AVCaptureOutput *)captureOutput
didOutputMetadataObjects:(NSArray *)metadataObjects
       fromConnection:(AVCaptureConnection *)connection
{
    NSString *QRCode = nil;
    [_session stopRunning];
    [_previewLayer removeFromSuperlayer];
    _session = nil;
    _previewLayer = nil;
    
    
    for (AVMetadataObject *metadata in metadataObjects) {
        if ([metadata.type isEqualToString:AVMetadataObjectTypeQRCode]) {
            // This will never happen; nobody has ever scanned a QR code... ever
            QRCode = [(AVMetadataMachineReadableCodeObject *)metadata stringValue];
            NSLog(@"QR Code: %@", QRCode);
            [[L7SClientManager sharedClientManager] authorize:QRCode failure:^(NSError *error){
                NSLog(@"failed to authenticate with QR code: %@", error);
                UIAlertView *alertView =   [[UIAlertView alloc] initWithTitle:@"QR code authentication"
                                                                      message:@"QR code authentication failed"
                                                                     delegate:nil
                                                            cancelButtonTitle:@"OK"
                                                            otherButtonTitles:nil];
                
                [alertView show];

            }];
            [_scanCodeButton setTitle:@"Scan Code"];
            break;
        }
    }
}


@end
