//
//  LSLoginViewController.m
//  sdkdemo
//
//  Created by Anthony Yu on 2013-09-26.
//  Copyright (c) 2013 CA Technologies. All rights reserved.
//

#import "LSLoginViewController.h"
#import "L7SClientManager.h"
#import "L7SAuthenticationProvider.h"


@interface LSLoginViewController () {
    
    NSArray *_socailLoginProviders;
    //the original view frame must be used to specify the view's width and height whenever appropriate in the "drawLoginDialogWithAuthProviders" implememtation.
    CGRect _originalViewFrame;
}

@end



@implementation LSLoginViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        _originalViewFrame = self.view.frame;
    }
    return self;
}

- (void) viewWillAppear:(BOOL)animated{
    
    [self.collectionView reloadData];
}

/**
 implement the method of L7SLoginDialogDelegate to support the social login.
 
 The implementation adjusts the login dilag view and its subviews based on the number and size of the soical login buttons, and return the view frame of the login dialog
 */
- (CGRect) drawLoginDialogWithAuthProviders: (L7SAuthenticationProvider *) authenticationProviders{
    _socailLoginProviders= [authenticationProviders getSocialLoginProviders];
    unsigned long count = _socailLoginProviders.count;
    if(count==0){
        if(![authenticationProviders isQRCodeAvailable]){
            //have a fixed height as the collectionView and QR code view are not needed
            return CGRectMake([self view].frame.origin.x, [self view].frame.origin.y, _originalViewFrame.size.width, 215);
        }else{
            //position the QR code frame
            self.qrCodeView.frame = CGRectMake(self.qrCodeView.frame.origin.x, 210, self.qrCodeView.frame.size.width, self.qrCodeView.frame.size.height);
            //draw the QR code
            [authenticationProviders loadQRCode:self.qrCodeView withPollDelay:10 pollInterval:5 maxPollCount:6];
            
            //adjust the height of the frame based on the line of cells
            return CGRectMake([self view].frame.origin.x, [self view].frame.origin.y, _originalViewFrame.size.width, _originalViewFrame.size.height +_qrCodeView.frame.size.height);
        }
        
    }else{
        //calculating the max number of button for each line.  The min space defined in the interface builder is 4
        UIButton *button = [_socailLoginProviders objectAtIndex:0];
        int lineCount= (count * (button.frame.size.width + 4)-4)/(_collectionView.frame.size.width)+1;
        
        //adjust of the size of the collection view
        _collectionView.frame = CGRectMake(_collectionView.frame.origin.x, _collectionView.frame.origin.y, _collectionView.frame.size.width, 50*lineCount);
        
        //adjust the QR code view accordingly
        if([authenticationProviders isQRCodeAvailable]){
            //position the QR code frame
            self.qrCodeView.frame = CGRectMake(self.qrCodeView.frame.origin.x, _collectionView.frame.origin.y+_collectionView.frame.size.height, self.qrCodeView.frame.size.width, self.qrCodeView.frame.size.height);
            //draw the QR code
            [authenticationProviders loadQRCode:self.qrCodeView withPollDelay:10 pollInterval:5 maxPollCount:6];
        }
        
        //adjust the height of the frame based on the line of cells.  The height grows based on the fixed height 215.
        return CGRectMake([self view].frame.origin.x, [self view].frame.origin.y, _originalViewFrame.size.width, 215 + (button.frame.size.height + 10)*lineCount+_qrCodeView.frame.size.height);
    }
    
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.

    [_collectionView setDataSource:self];
    [_collectionView setDelegate:self];
    
    [_collectionView registerClass:[UICollectionViewCell class] forCellWithReuseIdentifier:@"cellIdentifier"];
    [_collectionView setBackgroundColor:[UIColor whiteColor]];
  
}



- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)doLogin:(id)sender {
   
    [self.userNameField resignFirstResponder];
    [self.passwordField resignFirstResponder];
    
    
  [ [L7SClientManager sharedClientManager] authenticateWithUserName:self.userNameField.text password:self.passwordField.text];
    
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    NSLog(@"social login providers count: %lu", (unsigned long)_socailLoginProviders.count);
    return _socailLoginProviders.count;
}


- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    UICollectionViewCell *cell=[collectionView dequeueReusableCellWithReuseIdentifier:@"cellIdentifier" forIndexPath:indexPath];
    [[[cell subviews] objectAtIndex:0] removeFromSuperview];
    [cell addSubview:[_socailLoginProviders objectAtIndex:indexPath.row]];

    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
    UIButton *button = [_socailLoginProviders objectAtIndex:indexPath.row];
    return CGSizeMake(button.frame.size.width, button.frame.size.height);
}

@end
