//
//  L7SProximityAuthentication.h
//  L7SingleSignOnLibrary
//
//  Created by Anthony Yu on 2014-06-13.
//  Copyright (c) 2014 CA Technologies Technologies. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "UIImageView+L7SAFNetworking.h"

/**
This class encapsulates the authentication providers such as social login and QR code.
 */



@interface L7SAuthenticationProvider : NSObject


+ (instancetype)sharedInstance;

/**
  Load QR code by passing with an imageView and polling configuration.  The size of the QR image will be based on the width of the imageView.
  
 @param imageView the imageView that contains the QR code
 @param pollDelay the delay (measured in seconds) for the first poll.  Specify value of 0 or negative value will trigger an exception
 @param pollIntervals the interval (measured in seconds) between pools.  Specify value of 0 or negative value will trigger an exception
 @param maxPoll the maximum number of polls. Specify value of 0 or negative value will trigger an exception
*/
- (void) loadQRCode: (UIImageView *) imageView withPollDelay:(NSTimeInterval) pollDelay pollInterval:(NSTimeInterval) pollInterval maxPollCount:(int) maxPoll;

/**
  Load QR code by passing with an imageView.  The default polling configuration will be applied (deplay: 10 seconds, interval: 5 seconds, maxPoll: 6)
 */
- (void) loadQRCode: (UIImageView *) imageView;


/**
  Check if QR code is available
 */
- (BOOL) isQRCodeAvailable;

/**
  This method will remove the QR image from the login dialog, and stop the polling
 */
- (void) removeQRImage;

/**
  Get a list of social login providers
 */
- (NSArray *) getSocialLoginProviders;

@end
